#ifndef ROSETTE_H
#define ROSETTE_H


// Rosette.h
// Interfaces for the Rosette class and CreditsView class

// Rosettes, a screensaver module for BeOS
// Jon Pimble -- fatrat@fat-rat.com


#include <stdlib.h>
#include <sys/types.h>
#include <time.h>
#include <math.h>

#include <ScreenSaver.h>
#include <View.h>
#include <Font.h>
#include <Screen.h>


// Strings used in credits
#define CREDITS_SSNAME "Rosettes"
#define CREDITS_VERSION "Version 1.10 --- 2002.10.04"
#define CREDITS_AUTHOR "Jon Pimble - fatrat@fat-rat.com"


// Constant used to convert from degrees to radians
#define DEG2RAD (PI/180.0)

// Maximum number of rosettes allowed
#define MAX_FLOWERS 5

// Maximum number of petals on each rosette (minimum has to be 2)
#define MAX_PETALS 12

// Time to wait between drawing frames of animation (in microseconds)
#define TICKS ((1/60)*10000)


// Class is derived from the BScreenSaver base class
class Rosette : public BScreenSaver
{
  private:
  
    // HSV color value, with alpha component
    typedef struct
    {
      float h; // 0 to 360 (it's degrees)
      float s;
      float v;
      float alpha;
    } hsv_color;

    // Converts HSV color values to RGB
    rgb_color hsv_to_rgb(hsv_color hsv_to_convert);
    
    
    // Rosette structure
    typedef struct
    {
      // Position of center of rosette
      float x_pos, y_pos;
      float dx, dy;
    
      // Offset angle to rotate petals around center
      float petal_ofs_angle;
    
      // Radii for circles about which the petals are plotted
      float r1, r2, r3;
      float d_r1, d_r2, d_r3;
     
      // Number of petals
      int num_petals;

      // Degree measure of offset for middle petal coordinates
      //  (about the circle defined by r2)
      float petal_mid_ofs_angle;
    
      // HSV color (so we can select color via shifting hues...)
      hsv_color petal_color;
    } rosette;
    
    
    // Our rosettes
    rosette flowers[MAX_FLOWERS];
    
    // How many rosettes we're drawing
    int num_flowers;
    
  public:
    // Constructor
    Rosette( BMessage *archive, image_id image );    
    
    // Override virtual function implementations from base class
    void StartConfig( BView *view ); 
    status_t StartSaver( BView *view, bool preview ); 
    void Draw( BView *view, int32 frame );        
};



// Class for credits view
class CreditsView : public BView
{
  private:
      
  public:
    // Constructor
    CreditsView(BRect frame, const char *name);
    
    // Override these virtual functions inherited from BView
    void Draw(BRect r);
};



#endif
